/*
 * Copyright (c) 2021 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

#include <stdio.h>
#include <zephyr/kernel.h>
#include <string.h>
#include <modem/sms.h>
#include <modem/nrf_modem_lib.h>
#include <modem/lte_lc.h>
#include <zephyr/logging/log.h>
#include <zephyr/logging/log_ctrl.h>
#include <modem/modem_info.h>
#include <modem/pdn.h>

LOG_MODULE_REGISTER(SMS_sample, LOG_LEVEL_DBG);

struct modem_param_info modem_param;
uint64_t sms_time = 0;

/* Event handler for LTE link control */
static void lte_handler(const struct lte_lc_evt *const evt)
{	
	switch (evt->type) {
		case LTE_LC_EVT_NW_REG_STATUS:
			if(evt->nw_reg_status == LTE_LC_NW_REG_UICC_FAIL){
				LOG_INF("Not registered; UICC failure");
			}
			else{
				if(evt->nw_reg_status == LTE_LC_NW_REG_SEARCHING){
					LOG_INF("Not registered. Trying to attach or searching for an operator");
				}
				else if(evt->nw_reg_status == LTE_LC_NW_REG_REGISTRATION_DENIED){
					LOG_INF("Registration denied");
				}
				else if(evt->nw_reg_status == LTE_LC_NW_REG_NOT_REGISTERED){ 
					LOG_INF("Not registered. UE is not currently searching for an operator");
				}
				else if(evt->nw_reg_status == LTE_LC_NW_REG_UNKNOWN){
					LOG_INF("Unknown, for example out of LTE coverage");
				}
			}

			LOG_INF("Network registration code: %s", evt->nw_reg_status == LTE_LC_NW_REG_REGISTERED_HOME ? "Connected - home network" : "Connected - roaming");
			sms_time = k_uptime_get();
			break;

		case LTE_LC_EVT_RRC_UPDATE:
			LOG_INF("RRC mode: %s", evt->rrc_mode == LTE_LC_RRC_MODE_CONNECTED ? "Connected" : "Idle");
		
			
			break;
	
		default:
			break;
	}
}

static void sms_callback(struct sms_data *const data, void *context)
{
	if (data == NULL) {
		LOG_INF("%s with NULL data\n", __func__);
		return;
	}

	if (data->type == SMS_TYPE_DELIVER) {
		/* When SMS message is received, print information */
		struct sms_deliver_header *header = &data->header.deliver;


		LOG_WRN("Time to receive SMS: %2.1f s", (float)(k_uptime_get() - sms_time) / 1000.0);

		LOG_INF("\tSMS received:");
		LOG_INF("\tTime:   %02d-%02d-%02d %02d:%02d:%02d",
			header->time.year,
			header->time.month,
			header->time.day,
			header->time.hour,
			header->time.minute,
			header->time.second);

		LOG_INF("\tText:   '%s'", data->payload);
		LOG_INF("\tLength: %d", data->payload_len);

		if (header->app_port.present) {
			LOG_INF("\tApplication port addressing scheme: dest_port=%d, src_port=%d",
				header->app_port.dest_port,
				header->app_port.src_port);
		}
		if (header->concatenated.present) {
			LOG_INF("\tConcatenated short message: ref_number=%d, msg %d/%d",
				header->concatenated.ref_number,
				header->concatenated.seq_number,
				header->concatenated.total_msgs);
		}
	} 
	else if (data->type == SMS_TYPE_STATUS_REPORT) {
		LOG_INF("SMS status report received");
	} 
	else {
		LOG_INF("SMS protocol message with unknown type received");
	}
}


int main(void)
{
	// Variables
	int handle = 0;
	int ret = 0;
	int err = 0;
	enum lte_lc_system_mode mode;
	enum lte_lc_system_mode_preference preference;


	LOG_INF("SMS sample starting");


	// Turn ON modem
	ret = nrf_modem_lib_init();
	if (ret) {
		LOG_INF("Modem library initialization failed, error: %d", ret);
	}
	else{
		LOG_INF("Initialize the Modem library and turn on the modem");
	}

	if (lte_lc_func_mode_set(LTE_LC_FUNC_MODE_POWER_OFF) != 0) { 
		LOG_ERR("Failed to deactivate Modem");
	}

	// Modem Info init 
	err = modem_info_init();
	if(err != 0){
		LOG_ERR("Failed to initialize the modem information module");
	}
	else{
		LOG_INF("Initialize the modem information module");
	}
	err = modem_info_params_init(&modem_param);
	if(err != 0){
		LOG_ERR("Failed to initialize the struture that stores information");
	}
	else{
		LOG_INF("nitialize the structure that stores modem information");
	}


	// Set the modem system mode and LTE preference
	ret = lte_lc_system_mode_set(LTE_LC_SYSTEM_MODE_LTEM_GPS, LTE_LC_SYSTEM_MODE_PREFER_LTEM);
	if(ret != 0){
		LOG_ERR("Error lte_lc_system_mode_set, err: %d", ret);
	}
	ret = lte_lc_system_mode_get(&mode, &preference);
	if(ret != 0){
		LOG_ERR("Error lte_lc_system_mode_get, err: %d", ret);
	}
	else{
		LOG_INF("LTE mode = %d; LTE preference = %d", mode, preference);
	}

	

	char apn[20]="test";
	char default_apn[20];
	// Set APN
	err = pdn_ctx_configure(0, apn, PDN_FAM_IPV4, NULL);
	if(err){
		LOG_ERR("pdn_ctx_configure() failed, err %d", err);
		return 0;
	}
	else{
		LOG_INF("PDN context %d configured: APN: %s", 0, &apn[0]);
	}
	// Get default Access Point Name (APN)
	err = pdn_default_apn_get(default_apn, sizeof(default_apn));
	if(err != 0){
		LOG_ERR("Error %d pdn_default_apn_get", err);
	}
	else{
		LOG_INF("Default APN: %s", default_apn);
	}


	// Connect to LTE network
	lte_lc_register_handler(lte_handler);
	ret = lte_lc_connect();
	if (ret) {
		LOG_INF("Lte_lc failed to initialize and connect, err %d", ret);
		return 0;
	}


	// Register SMS callback
	handle = sms_register_listener(sms_callback, NULL);
	if (handle) {
		LOG_INF("sms_register_listener returned err: %d", handle);
	}
	LOG_INF("SMS sample is ready for receiving messages");

	// Send SMS
	if (strcmp(CONFIG_SMS_SEND_PHONE_NUMBER, "")) {
		LOG_INF("Sending SMS: number=%s, text=\"SMS sample: testing", CONFIG_SMS_SEND_PHONE_NUMBER);
		ret = sms_send_text(CONFIG_SMS_SEND_PHONE_NUMBER, "SMS sample: testing");
		if (ret) {
			LOG_INF("Sending SMS failed with error: %d\n", ret);
		}
	} 
	else{
		LOG_INF("\nSMS sending is skipped but receiving will still work.\n"
			"If you wish to send SMS, please configure CONFIG_SMS_SEND_PHONE_NUMBER\n");
	}


	/* Sending is done to the phone number specified in the configuration,
	 * or if it's left empty, an information text is printed.
	 */
	/*if (strcmp(CONFIG_SMS_SEND_PHONE_NUMBER, "")) {
		LOG_INF("Sending SMS: number=%s, text=\"SMS sample: testing", CONFIG_SMS_SEND_PHONE_NUMBER);
		ret = sms_send_text(CONFIG_SMS_SEND_PHONE_NUMBER, "SMS sample: testing");
		if (ret) {
			LOG_INF("Sending SMS failed with error: %d\n", ret);
		}
	} 
	else{
		LOG_INF("\nSMS sending is skipped but receiving will still work.\n"
			"If you wish to send SMS, please configure CONFIG_SMS_SEND_PHONE_NUMBER\n");
	}*/


	/* In our application, we should unregister SMS in some conditions with:
	 *   sms_unregister_listener(handle);
	 * However, this sample will continue to be registered for
	 * received SMS messages and they can be seen in serial port log.
	 */

    //sms_unregister_listener(sms_callback);
}
